"""
__version__.py
~~~~~~~~~~~~~~

Information about the current version of the pyDIOPT package.
"""

__title__ = 'pyDIOPT'
__description__ = 'pyDIOPT - Python Wrapper for DIOPT'
__version__ = '0.1.0'
__author__ = 'AZ Zhang'
__author_email__ = 'azz@berkeley.edu'
__license__ = 'Apache 2.0'
__url__ = 'https://github.com/akmazian/pyDIOPT'
