def citation() -> None:
    print("Please cite the original DIOPT papers")
    print(
        """
        Yanhui Hu, Ian Flockhart, Arunachalam Vinayagam, Clemens Bergwitz, Bonnie Berger, Norbert Perrimon, and Stephanie E Mohr. 2011. “An integrative approach to ortholog prediction for disease-focused and other functional studies.” BMC Bioinformatics, 12, Pp. 357.
        
        Yanhui Hu, Aram Comjean, Jonathan Rodiger, Yifang Liu, Yue Gao, Verena Chung, Jonathan Zirin, Norbert Perrimon, and Stephanie E Mohr. 2020. “FlyRNAi.org-the database of the Drosophila RNAi screening center and transgenic RNAi project: 2021 update.” Nucleic Acids Res.
        """
    )
    print()

    print("The links can be found here:")
    print(
        "[BMC Bioinformatics](https://fgr.hms.harvard.edu/publications/integrative-approach-ortholog-prediction-disease-focused-and-other-functional)"
    )
    print(
        "[Nucleic Acids Res](https://fgr.hms.harvard.edu/publications/flyrnaiorg-database-drosophila-rnai-screening-center-and-transgenic-rnai-project)"
    )

    print()

    print(
        "It would be appreciated if you also cited pyDIOPT :) though it is not required."
    )


def help() -> None:
    print("The documentations for pyDIOPT can be found at:")
    print("*not quite ready yet*")
    print()
    print("The documentations for DIOPT can be found at:")
    print("[DIOPT](https://fgr.hms.harvard.edu/diopt-documentation)")
